#! /usr/bin/bash

INIT_CGROUP="/sys/fs/cgroup/$(cut -d":" -f3 /proc/self/cgroup)"
ROOT_CGROUP="/sys/fs/cgroup/user.slice/user-1000.slice/user@1000.service/session.slice"
CGROUP_NAME=testcg

function pause() {
    echo -n "Press Enter to continue..."
    read
    echo
}

echo "Namespaces"
echo "========================================================================="
echo

echo "Run Bash in a new PID namespace"
echo -e "\twe need root rights to unshare PIDs"
echo -e "\t--fork tells unshare to fork before running Bash, so that Bash is the PID 1"
echo -e "\t--mount-proc tells unshare to mount the proc sysfs"
echo "-------------------------------------------------------------------------"
echo "# unshare --pid --fork --mount-proc ps"

sudo unshare --pid --fork --mount-proc ps

echo "-------------------------------------------------------------------------"
echo

pause

echo "List network interfaces in a new net namespace"
echo -e "\twe need root right to unshare network interfaces"
echo "-------------------------------------------------------------------------"
echo "# unshare --net ip link"

sudo unshare --net ip link

echo "-------------------------------------------------------------------------"
echo

pause

echo "Be root in a user namespace"
echo -e "\t--map-root-user (implies --user) tells unshare to map the current user to root"
echo "-------------------------------------------------------------------------"
echo "$ unshare --user --map-root-user whoami"

unshare --user --map-root-user whoami

echo "========================================================================="
echo

pause

echo "Control groups"
echo "========================================================================="
echo

echo "Check cgroup controllers enabled in sub cgroups"
echo "-------------------------------------------------------------------------"
echo -e "$ cd \"$ROOT_CGROUP\" &&\n\
\tcat cgroup.subtree_control"

pushd "$ROOT_CGROUP" >/dev/null &&\
    cat cgroup.subtree_control

echo "-------------------------------------------------------------------------"
echo

pause

echo "Create a new cgroup under the user's cgroup hierarchy"
echo "-------------------------------------------------------------------------"
echo -e "$ mkdir \"$CGROUP_NAME\" &&\n\
\tcd \"$CGROUP_NAME\""

mkdir "$CGROUP_NAME" &&\
    pushd "$CGROUP_NAME" >/dev/null || exit

echo "-------------------------------------------------------------------------"
echo

pause

echo "Check cgroup controllers enable in the current cgroup"
echo "$ cat cgroup.controllers"

cat cgroup.controllers

echo "-------------------------------------------------------------------------"
echo

pause

echo "Display maximum memory allocation authorized"
echo "-------------------------------------------------------------------------"
echo "$ cat memory.max"

cat memory.max

echo "-------------------------------------------------------------------------"
echo

pause

echo "Set maximum memory amount"
echo "-------------------------------------------------------------------------"
echo "$ echo 1073741824 > memory.max"

echo 1073741824 > memory.max

echo "-------------------------------------------------------------------------"
echo

pause

echo "Put the current process in the cgroup"
echo "-------------------------------------------------------------------------"
echo "$ echo \$\$ > cgroup.procs"

# To move a PID, we need write access to cgroup.procs of the common ancestor of
# the source cgroup and the destination cgroup.
sudo bash -c "echo $$ > cgroup.procs"

echo "-------------------------------------------------------------------------"
echo

pause

popd >/dev/null || exit

echo "Display CPU usage of a Docker container"
echo "-------------------------------------------------------------------------"
echo "$ cat \"$(find /sys/fs/cgroup/system.slice -name "docker-*" | head -n1)\"/cpu.stat"

cat "$(find /sys/fs/cgroup/system.slice -name "docker-*" | head -n1)"/cpu.stat

echo "-------------------------------------------------------------------------"
echo

pause

sudo bash -c "echo $$ > $INIT_CGROUP/cgroup.procs"
rmdir "$ROOT_CGROUP/$CGROUP_NAME"

